target_host=arm-none-eabi
standalone_toolchain=arm-none-eabi
cc_compiler=gcc
cxx_compiler=g++

[settings]
os=baremetal
arch=armv8
compiler=gcc
compiler.version=10
compiler.libcxx=libstdc++
compiler.cppstd={{ config.project.cxxstandard }}
arch_build=x86_64
build_type={{ config.profiles[profile].cmake_build_type }}

[env]
CONAN_CMAKE_FIND_ROOT_PATH=/opt/gcc-arm-none-eabi/lib/
CC=$target_host-$cc_compiler
CXX=$target_host-$cxx_compiler

[conf]
tools.build:cflags=["{{ config.project.cflags if config.project.cflags}} {{config.profiles.get(profile).cflags if config.profiles.get(profile).cflags}}"]
tools.build:cxxflags=["{{ config.project.cxxflags if config.project.cxxflags }} {{ config.profiles.get(profile).cxxflags if config.profiles.get(profile).cxxflags }}"]

{% if config.project.max_build_jobs != None %}
tools.build:jobs={{config.project.max_build_jobs}}
{% endif %}

[options]
[build_requires]
